// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

uniform float brightness;
uniform float specX;
uniform float specY;

void main()
         {
            vec3 Normal = normalize(N);
			vec3 Epsilon = vec3(1.0f, 0.0f, 0.0f);
            vec3 Tangent = normalize(cross(Normal, Epsilon));
 
            vec3 Viewer = -normalize(v);
            vec3 Light;
            float atten;
 
			for(int i = 0; i < 2; i++) {
			    // Directional
				if (i == 0)	{
					atten = 1.0; // no attenuation
					Light = normalize(gl_LightSource[i].position.xyz);
				// Other (Point) Light
				} else {
					vec3 Light_Pnt_Original = gl_LightSource[i].position.xyz - v;
					float dist = length(Light_Pnt_Original);
					atten = 1.0 / dist; 
					Light = normalize(Light_Pnt_Original);
				}
 
				vec3 Half = normalize(Light + Viewer);
				vec3 biNormal = cross(Normal, Tangent);
				float LdotN = dot(Light, Normal); 
 
				vec4 ambient = gl_FrontLightProduct[i].ambient;	 
	            vec4 diffuse = gl_FrontLightProduct[i].diffuse
								* atten * max(0.0, LdotN);	 
				vec4 specular;

				if (LdotN < 0.0) {
					specular = vec4(0.0, 0.0, 0.0, 1.0); 
				}
				else {
					float HdotN = dot(Half, Normal);
					float VdotN = dot(Viewer, Normal);
					float TdotHspecX = dot(Half, Tangent) / specX;
					float BdotHspecY = dot(Half, biNormal) / specY;
					
					float exp_temp = pow(TdotHspecX, 2) + pow(BdotHspecY, 2);
					vec4 coef_temp = atten * gl_LightSource[i].specular 
									  * sqrt(max(0.0, LdotN / VdotN));
 
			       specular =  coef_temp * exp(-2.0 * exp_temp / (1.0 + HdotN));
				}

				vec4 curr_Color = ambient + diffuse + specular;

				if(i == 0) gl_FragColor = curr_Color + gl_FrontMaterial.emission;
				else gl_FragColor += curr_Color;
			}

         }
